/******************************************************
                  DirectShow .NET
		      netmaster@swissonline.ch
*******************************************************/
//					DsDevice
// enumerate directshow devices and moniker handling


using System;
using System.Collections;
using System.Runtime.InteropServices;

namespace DShowNET.Device
{

	[ComVisible(false)]
	public class DsDev
	{

		public static bool GetDevicesOfCat( Guid cat, out ArrayList devs )
		{
			devs = null;
			int hr;
			object comObj = null;
			ICreateDevEnum enumDev = null;
			UCOMIEnumMoniker enumMon = null;
			UCOMIMoniker[] mon = new UCOMIMoniker[1];
			try 
			{
				Type	srvType = Type.GetTypeFromCLSID( Clsid.SystemDeviceEnum );
				if( srvType == null )
					throw new NotImplementedException( "System Device Enumerator" );

				comObj = Activator.CreateInstance( srvType );
				enumDev = (ICreateDevEnum) comObj;
				hr = enumDev.CreateClassEnumerator( ref cat, out enumMon, 0 );
				if( hr != 0 )
					throw new NotSupportedException( "No devices of the category" );

				int f, count = 0;
				do
				{
					hr = enumMon.Next( 1, mon, out f );
					if( (hr != 0) || (mon[0] == null) )
						break;
					DsDevice dev = new DsDevice();
					dev.Name = GetFriendlyName( mon[0] );
					if( devs == null )
						devs = new ArrayList();
					dev.Mon = mon[0]; mon[0] = null;
					devs.Add( dev ); dev = null;
					count++;
				}
				while(true);

				return count > 0;
			}
			catch( Exception )
			{
				if( devs != null )
				{
					foreach( DsDevice d in devs )
						d.Dispose();
					devs = null;
				}
				return false;
			}
			finally
			{
				enumDev = null;
				if( mon[0] != null )
					Marshal.ReleaseComObject( mon[0] ); mon[0] = null;
				if( enumMon != null )
					Marshal.ReleaseComObject( enumMon ); enumMon = null;
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}

		}

		public static bool GetDevicesByMatch( bool ExactMatch, Merit Merit, 
			bool InputNeeded, Guid[] InputTypes, Guid PinCategoryIn,
			bool Render,
			bool OutputNeeded, Guid[] OutputTypes, Guid PinCategoryOut,
			out ArrayList devs )
		{
			devs = null;
			int hr;
			object comObj = null;
			IFilterMapper2 mapDev = null;
			UCOMIEnumMoniker mapMon = null;
			UCOMIMoniker[] mon = new UCOMIMoniker[1];
			try 
			{
				Type	srvType = Type.GetTypeFromCLSID( Clsid.FilterMapper2 );
				if( srvType == null )
					throw new NotImplementedException( "System Device Enumerator" );

				comObj = Activator.CreateInstance( srvType );
				mapDev = (IFilterMapper2) comObj;
				hr = mapDev.EnumMatchingFilters( out mapMon, 0, ExactMatch, Merit, 
					InputNeeded, (InputTypes != null) ? (InputTypes.Length / 2) : 0, InputTypes, 0, ref PinCategoryIn, 
					Render, 
					OutputNeeded, (OutputTypes != null) ? (OutputTypes.Length / 2) : 0, OutputTypes, 0, ref PinCategoryOut);
				if( hr != 0 )
					throw new NotSupportedException( "No devices match the filter" );

				int f, count = 0;
				do
				{
					hr = mapMon.Next( 1, mon, out f );
					if( (hr != 0) || (mon[0] == null) )
						break;
					DsDevice dev = new DsDevice();
					dev.Name = GetFriendlyName( mon[0] );
					if( devs == null )
						devs = new ArrayList();
					dev.Mon = mon[0]; mon[0] = null;
					devs.Add( dev ); dev = null;
					count++;
				}
				while(true);

				return count > 0;
			}
			catch( Exception )
			{
				if( devs != null )
				{
					foreach( DsDevice d in devs )
						d.Dispose();
					devs = null;
				}
				return false;
			}
			finally
			{
				mapDev = null;
				if( mon[0] != null )
					Marshal.ReleaseComObject( mon[0] ); mon[0] = null;
				if( mapMon != null )
					Marshal.ReleaseComObject( mapMon ); mapMon = null;
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}

		}

		private static string GetFriendlyName( UCOMIMoniker mon )
		{
			object bagObj = null;
			IPropertyBag bag = null;
			try 
			{
				Guid bagId = typeof( IPropertyBag ).GUID;
				mon.BindToStorage( null, null, ref bagId, out bagObj );
				bag = (IPropertyBag) bagObj;
				object val = "";
				int hr = bag.Read( "FriendlyName", ref val, IntPtr.Zero );
				if( hr != 0 )
					Marshal.ThrowExceptionForHR( hr );
				string ret = val as string;
				if( (ret == null) || (ret.Length < 1) )
					throw new NotImplementedException( "Device FriendlyName" );
				return ret;
			}
			catch( Exception )
			{
				return null;
			}
			finally
			{
				bag = null;
				if( bagObj != null )
					Marshal.ReleaseComObject( bagObj ); bagObj = null;
			}
		}
	}


	[ComVisible(false)]
	public class DsDevice : IDisposable
	{
		public string			Name;
		private string			displayName;
		public UCOMIMoniker		Mon;
		
		public void Dispose()
		{
			if( Mon != null )
				Marshal.ReleaseComObject( Mon ); Mon = null;
		}

		public string GetDisplayName()
		{
			if (displayName == null)
			{
				Mon.GetDisplayName(null, null, out displayName);
			}
			return displayName;
		}

		public object BindToObject(Type interfaceType)
		{
			object vResult;
			Guid iidResult = interfaceType.GUID;
			Mon.BindToObject(null, null, ref iidResult, out vResult);
			return vResult;
		}

		public override string ToString()
		{
			return Name;
		}
	}




	[ComVisible(true), ComImport,
	Guid("B79BB0B0-33C1-11D1-ABE1-00A0C905F375"),
	InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	public interface IFilterMapper2
	{
		[PreserveSig]
		int  CreateCategory(
			[In]										ref Guid				clsidCategory, 
			[In]											int					dwCategoryMerit,
			[In, MarshalAs(UnmanagedType.LPWStr)]			string				Description);

		[PreserveSig]
		int  UnregisterFilter(
			[In]										ref	Guid				pclsidCategory,
			[In, MarshalAs(UnmanagedType.LPWStr)]			string				szInstance, 
			[In]										ref Guid				clsidFilter);
		
		[PreserveSig]
		int  RegisterFilter(
			[In]										ref Guid				clsidFilter, 
			[In, MarshalAs(UnmanagedType.LPWStr)]			string				Name,
			[Out]										out UCOMIEnumMoniker	ppMoniker, 
			[In]										ref	Guid				pclsidCategory,
			[In, MarshalAs(UnmanagedType.LPWStr)]			string				szInstance, 
			[In]											int					prf2);

		[PreserveSig]
		int EnumMatchingFilters(
			[Out]															out UCOMIEnumMoniker	ppEnumMoniker,
			[In]																int					dwFlags,
			[In, MarshalAs(UnmanagedType.Bool)]									bool				bExactMatch,
			[In]																Merit				dwMerit,
			[In, MarshalAs(UnmanagedType.Bool)]									bool				bInputNeeded,
			[In]																int					cInputTypes,
			[In, MarshalAs(UnmanagedType.LPArray)]								Guid[]				pInputTypes,
			[In]																int					pMedIn,
			[In]															ref	Guid				pPinCategoryIn,
			[In, MarshalAs(UnmanagedType.Bool)]									bool				bRender,
			[In, MarshalAs(UnmanagedType.Bool)]									bool				bOutputNeeded,
			[In]																int					cOutputTypes,
			[In, MarshalAs(UnmanagedType.LPArray)]								Guid[]				pOutputTypes,
			[In]																int					pMedOut,
			[In]															ref	Guid				pPinCategoryOut);
	}




	[ComVisible(true), ComImport,
	Guid("29840822-5B84-11D0-BD3B-00A0C911CE86"),
	InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	public interface ICreateDevEnum
	{
		[PreserveSig]
		int CreateClassEnumerator(
			[In]											ref Guid			pType,
			[Out]										out UCOMIEnumMoniker	ppEnumMoniker,
			[In]											int					dwFlags );
	}



	[ComVisible(true), ComImport,
	Guid("55272A00-42CB-11CE-8135-00AA004BB851"),
	InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	public interface IPropertyBag
	{
		[PreserveSig]
		int Read(
			[In, MarshalAs(UnmanagedType.LPWStr)]			string			pszPropName,
			[In, Out, MarshalAs(UnmanagedType.Struct)]	ref	object			pVar,
			IntPtr pErrorLog );

		[PreserveSig]
		int Write(
			[In, MarshalAs(UnmanagedType.LPWStr)]			string			pszPropName,
			[In, MarshalAs(UnmanagedType.Struct)]		ref	object			pVar );
	}


	[ComVisible(false)]
	public enum Merit
	{
		Preferrer       = 0x800000,	// MERIT_PREFERRED
		Normal          = 0x600000,	// MERIT_NORMAL
		Unlikely        = 0x400000,	// MERIT_UNLIKELY
		Do_Not_Use      = 0x200000,	// MERIT_DO_NOT_USE
		SW_Compressor   = 0x100000,	// MERIT_SW_COMPRESSOR
		HW_Compressor   = 0x100050	// MERIT_HW_COMPRESSOR
	}


} // namespace DShowNET.Device
