using System;
using System.Runtime.InteropServices;

namespace DShowNET.Moonlight
{

	[ComVisible(false)]
	public class Moonlight
	{

		public static void ConfigureMoonlightMux(IMoonlightConfig muxcfg)
		{
			Guid api;
			int hr;
			object comObj;

			// Change OutputStreamType to Program
			api = M71.OutputStreamType;
			comObj = OutputStreamType.Program;
			hr = muxcfg.SetValue(ref api, ref comObj);
			if( hr != 0 )
				Marshal.ThrowExceptionForHR( hr );

			// Get First Program
			api = M71.GetNextProgram;
			hr = muxcfg.GetValue(ref api, out comObj);
			if( hr != 0 )
				Marshal.ThrowExceptionForHR( hr );

			// Create a new input Stream for the Program
			IMoonlightConfig program = comObj as IMoonlightConfig;
			if (program == null) return;

			api = M71.CreateStream;
			hr = program.SetValue(ref api, ref comObj);
			if( hr != 0 )
				Marshal.ThrowExceptionForHR( hr );
		}

	}

	[ComVisible(true), ComImport,
	Guid("08B81CB2-3380-4994-A4F3-F0B6E99747F4"),
	InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	public interface IMoonlightConfig
	{
		#region "IPersist Methods"
		[PreserveSig]
		int GetClassID(
			[Out]									out Guid		pClassID );
		#endregion

		#region "IPersistStream Methods"
		[PreserveSig]
		void IsDirty(); // wrong but allocates space for the interface methods
		[PreserveSig]
		void Load(); // wrong but allocates space for the interface methods
		[PreserveSig]
		void Save(); // wrong but allocates space for the interface methods
		[PreserveSig]
		void GetSizeMax(); // wrong but allocates space for the interface methods
		#endregion

		int	GetDefaultValue(
			[In]									ref	Guid		Api,
			[Out]									out	object		pValue);

		int	GetParameterRange(
			[In]									ref	Guid		Api,
			[Out]									out	object		pValueMin,
			[Out]									out	object		pValueMax,
			[Out]									out	object		pSteppingDelta);
		
		int	GetValue(
			[In]									ref	Guid		Api,
			[Out]									out	object		pValue);
		
		int	IsModifiable(
			[In]									ref	Guid		Api);
		
		int	IsSupported(
			[In]									ref	Guid		Api);
		
		int	SetAllDefaults();
		
		int	SetValue(
			[In]									ref	Guid		Api,
			[In]									ref	object		pValue);
		
		int	ValidateAndApply(
			ref	object		pReason);
		
		int	SaveToRegistry(
			[In]										int			hKeyRoot,
			[In]										string		pszKeyName);
		
		int	LoadFromRegistry(
			[In]										int			hKeyRoot,
			[In]										string		pszKeyName);

	}
	
	[ComVisible(false)]
	public class M71		// PropID_M71.h  :  M71_*
	{
		/// <summary> M71_OutputStreamType </summary>
		public static readonly Guid OutputStreamType		= new Guid( "{0x8e30871a,0xbd18,0x46c1,{0x9e,0xce,0xbb,0x14,0x46,0xb0,0xce,0xd9}}" );

		/// <summary> M71_Bitrate </summary>
		public static readonly Guid Bitrate					= new Guid( "{0x296dc74a,0x417b,0x49e8,{0xa6,0xb5,0xfc,0xbf,0x52,0xc8,0x70,0x73}}" );

		/// <summary> M71_TransportStreamID </summary>
		public static readonly Guid TransportStreamID		= new Guid( "{0x9567e931,0x8d38,0x48d6,{0x80,0x5d,0x41,0x2f,0xc8,0xb1,0xab,0x59}}" );

		/// <summary> M71_GetNextProgram </summary>
		public static readonly Guid GetNextProgram			= new Guid( "{0xb68af389,0xed48,0x4e8e,{0x80,0x38,0x8a,0xeb,0xc9,0xba,0x6d,0xce}}" );

		/// <summary> M71_CreateProgram </summary>
		public static readonly Guid CreateProgram			= new Guid( "{0xf94d23b3,0xfa31,0x45b6,{0xa0,0x52,0xf,0xd9,0x6a,0xe8,0x45,0x1c}}" );

		/// <summary> M71_DestroyProgram </summary>
		public static readonly Guid DestroyProgram			= new Guid( "{0xc3b58da7,0x2a95,0x4d0f,{0x86,0x3,0xc2,0xa,0xa1,0x5,0x47,0xff}}" );

		/// <summary> M71_ProgramID </summary>
		public static readonly Guid ProgramID				= new Guid( "{0x1200a7ed,0x9902,0x4bec,{0x86,0x1,0xb,0x15,0xb6,0xae,0xc7,0xa5}}" );

		/// <summary> M71_ProgramNumber </summary>
		public static readonly Guid ProgramNumber			= new Guid( "{0x291d007d,0x1d9c,0x4f76,{0xa8,0xfa,0x15,0x2e,0xcb,0x55,0xa6,0xda}}" );

		/// <summary> M71_GetNextStream </summary>
		public static readonly Guid GetNextStream			= new Guid( "{0x7add47a1,0xd3e9,0x4ffe,{0x88,0x18,0x8e,0x15,0x15,0x1a,0x6c,0x91}}" );

		/// <summary> M71_CreateStream </summary>
		public static readonly Guid CreateStream			= new Guid( "{0xc91eaaa1,0xbb29,0x4eaa,{0xa7,0xa5,0x19,0x3f,0x25,0x30,0x6c,0x69}}" );

		/// <summary> M71_DestroyStream </summary>
		public static readonly Guid DestroyStream			= new Guid( "{0x36f3fbbc,0xe773,0x4f73,{0xa8,0xf8,0xda,0xbd,0x6c,0xeb,0x82,0x4b}}" );

		/// <summary> M71_StreamType </summary>
		public static readonly Guid StreamType				= new Guid( "{0x9b30cd08,0xf3db,0x4361,{0x8e,0xf9,0xaa,0xac,0xab,0x2a,0x9e,0xe7}}" );

		/// <summary> M71_PESPacketSize </summary>
		public static readonly Guid PESPacketSize			= new Guid( "{0x7900eca5,0xd886,0x4baa,{0x87,0x59,0x3d,0xeb,0x63,0xd4,0x27,0xfd}}" );

		/// <summary> M71_CarriagePCR </summary>
		public static readonly Guid CarriagePCR				= new Guid( "{0x55f19e,0x2fa7,0x47ab,{0xa6,0x79,0x10,0xbf,0x25,0x11,0x53,0xb4}}" );

	}

	[ComVisible(false)]
	public enum OutputStreamType
	{
		Program = 1,
		Transport = 2
	}

}
