using System;
using System.Collections;
using System.Runtime.InteropServices;

namespace DShowNET
{

	[ComVisible(false)]
	public class WmMixer
	{
		private int idMixer;
		private WmMIXERCAPS caps;

		internal WmMixer(int idMixer)
		{
			this.idMixer = idMixer;
			int hr = WmUtils.mixerGetDevCaps(idMixer, ref caps, Marshal.SizeOf(typeof(WmMIXERCAPS)));
			if( hr != 0 )
				WmError.ThrowExceptionForWM( hr );
		}

		public static ArrayList GetMixers()
		{
			ArrayList mixers = null;
			int numberOfMixers = WmUtils.mixerGetNumDevs();

			if (numberOfMixers > 0)
				mixers = new ArrayList();

			for (int i = 0; i < numberOfMixers; i++)
			{
				mixers.Add(new WmMixer(i));
			}

			return mixers;
		}

		public string Name
		{
			get
			{
				return caps.szPname;
			}
		}

		public ArrayList GetLines()
		{
			ArrayList lines = caps.cDestinations > 0 ? lines = new ArrayList() : null;

			for (int i = 0; i < caps.cDestinations; i++) 
			{
				lines.Add(new WmLine(idMixer, i));
			}

			return lines;
		}
	}

	[ComVisible(false)]
	public class WmLine 
	{
		private int idMixer;
		private int idLine;
		private WmMIXERLINE mixerLine = new WmMIXERLINE();

		internal WmLine(int idMixer, int idLine)
		{
			this.idMixer = idMixer;
			this.idLine = idLine;
			mixerLine.cbStruct = Marshal.SizeOf(typeof(WmMIXERLINE));
			mixerLine.dwDestination = idLine;
			int hr = WmUtils.mixerGetLineInfo(idMixer, ref mixerLine, 0);
			if( hr != 0 )
				WmError.ThrowExceptionForWM( hr );
		}

		public string Name
		{
			get
			{
				return mixerLine.szName;
			}
		}

		public WmTarget Target
		{
			get
			{
				return new WmTarget(mixerLine.Target);
			}
		}

		public ArrayList GetSources()
		{
			ArrayList sources = mixerLine.cConnections > 0 ? sources = new ArrayList() : null;

			for (int i = 0; i < mixerLine.cConnections; i++) 
			{
				sources.Add(new WmSource(idMixer, idLine, i));
			}

			return sources;
		}
	}
	
	[ComVisible(false)]
	public class WmSource 
	{
		private int idMixer;
		private int idLine;
		private WmMIXERLINE mixerLine = new WmMIXERLINE();

		internal WmSource(int idMixer, int idLine, int idSource)
		{
			this.idMixer = idMixer;
			this.idLine = idLine;
			mixerLine.cbStruct = Marshal.SizeOf(typeof(WmMIXERLINE));
			mixerLine.dwDestination = idLine;
			mixerLine.dwSource = idSource;
			int hr = WmUtils.mixerGetLineInfo(idMixer, ref mixerLine, 1);
			if( hr != 0 )
				WmError.ThrowExceptionForWM( hr );
		}

		public string Name
		{
			get
			{
				return mixerLine.szName;
			}
		}

		public WmTarget Target
		{
			get
			{
				return new WmTarget(mixerLine.Target);
			}
		}

		public ArrayList GetLineControls()
		{
			ArrayList lineControls = mixerLine.cControls > 0 ? new ArrayList() : null;
			IntPtr pMixerControls = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(WmMIXERCONTROL)) * mixerLine.cControls);
			try
			{

				WmMIXERLINECONTROLS mixerLineControls = new WmMIXERLINECONTROLS();
				mixerLineControls.cbStruct = Marshal.SizeOf(typeof(WmMIXERLINECONTROLS));
				mixerLineControls.dwLineID = mixerLine.dwLineID;
				mixerLineControls.cControls = mixerLine.cControls;
				mixerLineControls.cbmxctrl = Marshal.SizeOf(typeof(WmMIXERCONTROL));
				mixerLineControls.pamxctrl = pMixerControls;
				int hr = WmUtils.mixerGetLineControls(idLine, ref mixerLineControls, 0);
				if( hr != 0 )
					WmError.ThrowExceptionForWM( hr );

				IntPtr eMixerControls = pMixerControls;
				for (int i = 0; i < mixerLine.cControls; i++) 
				{
					lineControls.Add(new WmLineControl(idMixer, (WmMIXERCONTROL)Marshal.PtrToStructure(eMixerControls, typeof(WmMIXERCONTROL))));
					eMixerControls = new IntPtr(eMixerControls.ToInt64() + Marshal.SizeOf(typeof(WmMIXERCONTROL)));
				}
			}
			finally
			{
				Marshal.FreeHGlobal(pMixerControls);
			}

			return lineControls;
		}
	}

	[ComVisible(false)]
	public class WmTarget
	{
		private WmMIXERLINETarget mixerLineTarget;
		internal WmTarget(WmMIXERLINETarget mixerLineTarget)
		{
			this.mixerLineTarget = mixerLineTarget;
		}

		public string Name
		{
			get
			{
				return mixerLineTarget.szPname;
			}
		}

		public MixerLine_TargetType Type
		{
			get
			{
				return (MixerLine_TargetType)mixerLineTarget.dwType;
			}
		}
	
	}

	[ComVisible(false)]
	public class WmLineControl 
	{
		private int idMixer;
		private WmMIXERCONTROL mixerControl;

		internal WmLineControl(int idMixer, WmMIXERCONTROL mixerControl)
		{
			this.idMixer = idMixer;
			this.mixerControl = mixerControl;
		}

		public string Name
		{
			get
			{
				return mixerControl.szName;
			}
		}

		public MixerControl_ControlType ControlType
		{
			get
			{
				return (MixerControl_ControlType)mixerControl.dwControlType;
			}
		}

		public object Value 
		{
			get
			{
				return false;
			}
			set
			{
				WmMIXERCONTROLDETAILS details =  new WmMIXERCONTROLDETAILS();
				details.cbStruct = Marshal.SizeOf(typeof(WmMIXERCONTROLDETAILS));
				details.dwControlID = mixerControl.dwControlID;
				details.cChannels = 1;
				
				object ddetails = null;
				switch ((MixerControl_CT)(mixerControl.dwControlType & (int)MixerControl_CT.Units_Mask)) 
				{
					case MixerControl_CT.Units_Boolean:
						WmMIXERCONTROLDETAILS_BOOLEAN details_bool = new WmMIXERCONTROLDETAILS_BOOLEAN();
						details_bool.fValue = (bool)value;
						ddetails = details_bool;
						break;
					case MixerControl_CT.Units_Signed:
						WmMIXERCONTROLDETAILS_SIGNED details_signed = new WmMIXERCONTROLDETAILS_SIGNED();
						details_signed.lValue = (int)value;
						ddetails = details_signed;
						break;
					case MixerControl_CT.Units_Unsigned:
						WmMIXERCONTROLDETAILS_UNSIGNED details_unsigned = new WmMIXERCONTROLDETAILS_UNSIGNED();
						details_unsigned.lValue = (int)value;
						ddetails = details_unsigned;
						break;
				}

				int cbDetails = Marshal.SizeOf(ddetails);
				IntPtr paDetails = Marshal.AllocHGlobal(cbDetails);
				try 
				{
					Marshal.StructureToPtr(ddetails, paDetails, true);

					details.cbDetails = cbDetails;
					details.paDetails = paDetails;

					int hr = WmUtils.mixerSetControlDetails(idMixer, ref details, 0);
					if( hr != 0 )
						WmError.ThrowExceptionForWM( hr );
				}
				finally 
				{
					Marshal.Release(paDetails);
				}

			}
		}
	}
	
	public enum MixerLine_TargetType
	{
		Undefined = 0,
		WaveOut   = 1,
		WaveIn    = 2,
		MidiOut   = 3,
		MidiIn    = 4,
		Aux       = 5,
	}

	[Flags]
	enum MixerControl_CT : long 
	{
		Class_Mask		   = 0Xf0000000L,
		Class_Custom       = 0X00000000L,
		Class_Meter        = 0X10000000L,
		Class_Switch       = 0X20000000L,
		Class_Number       = 0X30000000L,
		Class_Slider       = 0X40000000L,
		Class_Fader        = 0X50000000L,
		Class_Time         = 0X60000000L,
		Class_List         = 0X70000000L,
							
		Subclass_Mask      = 0X0F000000L,
							
		Sc_Switch_Boolean  = 0X00000000L,
		Sc_Switch_Button   = 0X01000000L,
							
		Sc_Meter_Polled    = 0X00000000L,
							
		Sc_Time_Microsecs  = 0X00000000L,
		Sc_Time_Millisecs  = 0X01000000L,
							
		Sc_List_Single     = 0X00000000L,
		Sc_List_Multiple   = 0X01000000L,
							
		Units_Mask         = 0X00Ff0000L,
		Units_Custom       = 0X00000000L,
		Units_Boolean      = 0X00010000L,
		Units_Signed       = 0X00020000L,
		Units_Unsigned     = 0X00030000L,
		Units_Decibels     = 0X00040000L, /* In 10Ths */
		Units_Percent      = 0X00050000L, /* In 10Ths */
	}

	public enum MixerControl_ControlType : long
	{
		Custom         = (MixerControl_CT.Class_Custom | MixerControl_CT.Units_Custom),
		Booleanmeter   = (MixerControl_CT.Class_Meter | MixerControl_CT.Sc_Meter_Polled | MixerControl_CT.Units_Boolean),
		Signedmeter    = (MixerControl_CT.Class_Meter | MixerControl_CT.Sc_Meter_Polled | MixerControl_CT.Units_Signed),
		Peakmeter      = (Signedmeter + 1),
		Unsignedmeter  = (MixerControl_CT.Class_Meter | MixerControl_CT.Sc_Meter_Polled | MixerControl_CT.Units_Unsigned),
		Boolean        = (MixerControl_CT.Class_Switch | MixerControl_CT.Sc_Switch_Boolean | MixerControl_CT.Units_Boolean),
		Onoff          = (Boolean + 1),
		Mute           = (Boolean + 2),
		Mono           = (Boolean + 3),
		Loudness       = (Boolean + 4),
		Stereoenh      = (Boolean + 5),
		Bass_Boost     = (Boolean + 0X00002277),
		Button         = (MixerControl_CT.Class_Switch | MixerControl_CT.Sc_Switch_Button | MixerControl_CT.Units_Boolean),
		Decibels       = (MixerControl_CT.Class_Number | MixerControl_CT.Units_Decibels),
		Signed         = (MixerControl_CT.Class_Number | MixerControl_CT.Units_Signed),
		Unsigned       = (MixerControl_CT.Class_Number | MixerControl_CT.Units_Unsigned),
		Percent        = (MixerControl_CT.Class_Number | MixerControl_CT.Units_Percent),
		Slider         = (MixerControl_CT.Class_Slider | MixerControl_CT.Units_Signed),
		Pan            = (Slider + 1),
		Qsoundpan      = (Slider + 2),
		Fader          = (MixerControl_CT.Class_Fader | MixerControl_CT.Units_Unsigned),
		Volume         = (Fader + 1),
		Bass           = (Fader + 2),
		Treble         = (Fader + 3),
		Equalizer      = (Fader + 4),
		Singleselect   = (MixerControl_CT.Class_List | MixerControl_CT.Sc_List_Single | MixerControl_CT.Units_Boolean),
		Mux            = (Singleselect + 1),
		Multipleselect = (MixerControl_CT.Class_List | MixerControl_CT.Sc_List_Multiple | MixerControl_CT.Units_Boolean),
		Mixer          = (Multipleselect + 1),
		Microtime      = (MixerControl_CT.Class_Time | MixerControl_CT.Sc_Time_Microsecs | MixerControl_CT.Units_Unsigned),
		Millitime      = (MixerControl_CT.Class_Time | MixerControl_CT.Sc_Time_Millisecs | MixerControl_CT.Units_Unsigned),
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCAPS 
	{
		public	short	wMid;                   /* manufacturer id */
		public	short	wPid;                   /* product id */
		public	int		vDriverVersion;         /* version of the driver */
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst=WmUtils.MAXPNAMELEN)]
		public	string	szPname;				/* product name */
		public	int		fdwSupport;             /* misc. support bits */
		public	int		cDestinations;          /* count of destinations */
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERLINE
	{
		public	int		cbStruct;               /* size of MIXERLINE structure */
		public	int		dwDestination;          /* zero based destination index */
		public	int		dwSource;               /* zero based source index (if source) */
		public	int		dwLineID;               /* unique line id for mixer device */
		public	int		fdwLine;                /* state/information about line */
		public	int		dwUser;                 /* driver specific information */
		public	int		dwComponentType;        /* component type line connects to */
		public	int		cChannels;              /* number of channels line supports */
		public	int		cConnections;           /* number of connections [possible] */
		public	int		cControls;              /* number of controls at this line */
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst=WmUtils.MIXER_SHORT_NAME_CHARS)]
		public	string	szShortName;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst=WmUtils.MIXER_LONG_NAME_CHARS)]
		public	string	szName;

		public	WmMIXERLINETarget	Target;
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERLINETarget
	{
	
		public	int		dwType;                 /* MIXERLINE_TARGETTYPE_xxxx */
		public	int		dwDeviceID;             /* target device ID of device type */
		public	short	wMid;                   /* of target device */
		public	short	wPid;                   /*      " */
		public	int		vDriverVersion;         /*      " */
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst=WmUtils.MAXPNAMELEN)]
		public	string	szPname;				/*      " */
	};
	
	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERLINECONTROLS 
	{
		public	int					cbStruct;
		public	int					dwLineID;
		public	int					dwControlIDorType;
		public	int					cControls;
		public	int					cbmxctrl;
		public	IntPtr				pamxctrl;
	};

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCONTROL 
	{
		public	int				cbStruct;           /* size in bytes of MIXERCONTROL */
		public	int				dwControlID;        /* unique control id for mixer device */
		public	int				dwControlType;      /* xxx */
		public	int				fdwControl;         /* MIXERCONTROL_CONTROLF_xxx */
		public	int				cMultipleItems;     /* if MIXERCONTROL_CONTROLF_MULTIPLE set */
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst=WmUtils.MIXER_SHORT_NAME_CHARS)]
		public	string			szShortName;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst=WmUtils.MIXER_LONG_NAME_CHARS)]
		public	string			szName;
		WmMIXERCONTROLBounds	Bounds;
		WmMIXERCONTROLMetrics	Metrics;
	};

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCONTROLBounds
	{
		public	int			dwMinimum;           /* signed minimum for this control */
		public	int			dwMaximum;           /* signed maximum for this control */
		[MarshalAs(UnmanagedType.ByValArray, SizeConst=4)]
		public	int[]		dwReserved;
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCONTROLMetrics
	{
		public	int			cStepsOrCustomData;             /* # of steps between min & max */
		[MarshalAs(UnmanagedType.ByValArray, SizeConst=5)]
		public	int[]		dwReserved;      /* !!! needed? we have cbStruct.... */
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCONTROLDETAILS 
	{
		public	int			cbStruct;       /* size in bytes of MIXERCONTROLDETAILS */
		public	int			dwControlID;    /* control id to get/set details on */
		public	int			cChannels;      /* number of channels in paDetails array */
		public	int			cMultipleItems; /* if _MULTIPLE, the number of items per channel */
		public	int			cbDetails;      /* size of _one_ details_XX struct */
		public	IntPtr		paDetails;      /* pointer to array of details_XX structs */
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCONTROLDETAILS_BOOLEAN
	{
		public	bool		fValue;
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCONTROLDETAILS_SIGNED
	{
		public	int			lValue;
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCONTROLDETAILS_UNSIGNED
	{
		public	int			lValue;
	}

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
	struct WmMIXERCONTROLDETAILS_LISTTEXT
	{
		public	int			dwParam1;
		public	int			dwParam2;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst=WmUtils.MIXER_LONG_NAME_CHARS)]
		public	string		szName;
	}

	[ComVisible(false)]
	class WmUtils 
	{

		internal const int MAXPNAMELEN				= 32;
		internal const int MIXER_SHORT_NAME_CHARS	= 16;
		internal const int MIXER_LONG_NAME_CHARS	= 64;

		[DllImport("winmm")]
		internal static extern int mixerGetNumDevs();
		/*
		[DllImport("winmm")]
		static extern int mixerOpen( out int phmx, int uMxId, IntPtr dwCallback, IntPtr dwInstance, int fdwOpen);

		[DllImport("winmm")]
		static extern int mixerClose( int hmx);
		*/
		[DllImport("winmm")]
		internal static extern int mixerGetDevCaps(int uMxId, ref WmMIXERCAPS pmxcaps, int cbmxcaps);

		[DllImport("winmm")]
		internal static extern int mixerGetLineInfo(int hmxobj, ref WmMIXERLINE pmxl, int fdwInfo);

		[DllImport("winmm")]
		internal static extern int mixerGetLineControls(int hmxobj, ref WmMIXERLINECONTROLS pmxlc, int fdwControls);

		[DllImport("winmm")]
		internal static extern int mixerSetControlDetails( int hmxobj, ref WmMIXERCONTROLDETAILS pmxcd, int fdwDetails);
	}
}
