#region using
using System;
using System.Xml.Serialization;

using RecorderHelper;
#endregion

namespace SoftwareRecorder
{

	#region Config Enums

	public enum ChannelChangeOperation
	{
		None = 0,
		Pause = 1,
		Stop = 2,
		StopWhenReady = 3
	}

	// This should be change in GBPVR.Public.ScheduledRecording from constants to an enum
	public enum ScheduledRecording_Quality
	{
		High = 0,
		Medium = 1,
		Low = 2,
		Custom1 = 3,
		Custom2 = 4,
		LiveTV = 5,
		LiveTVPreview = 99
	}

	#endregion

	#region Quality Config

	[XmlType("quality")]
	public class Quality
	{

		/// <summary> the name of the quality. </summary>
		[XmlAttribute("profile"),XmlHashKey()]
		internal string					profileName;

		/// <summary> the color space of the picture. </summary>
		internal string					colorSpace = string.Empty;

		/// <summary> the width of the picture. </summary>
		internal int					width = 320;

		/// <summary> the height of the picture. </summary>
		internal int					height = 240;

	}

	#endregion

	#region Guid Helper
	public class GuidHelper
	{
		public static string GetGuidName(Type guidType, Guid value)
		{
			if (value == Guid.Empty) return "{Empty}";
			System.Reflection.FieldInfo[] fields = guidType.GetFields();
			foreach (System.Reflection.FieldInfo field in fields)
			{
				if ((Guid)field.GetValue(null) == value)
				{
					return field.Name;
				}
			}
			return "{" + value.ToString() + "}";
		}

		public static Guid GetGuidValue(Type guidType, string name)
		{
			if (name == "{Empty}") return Guid.Empty;
			if (name.StartsWith("{") && name.EndsWith("}"))
			{
                return new Guid(name);
			}
			System.Reflection.FieldInfo[] fields = guidType.GetFields();
			foreach (System.Reflection.FieldInfo field in fields)
			{
				if (field.Name == name)
				{
					return (Guid)field.GetValue(null);
				}
			}
			return Guid.Empty;
		}

	}
	#endregion

}
